#!/usr/bin/python
#
# Collect metadata from each data set.
#
# Important: this script relies on the COMMENT LINE generated by our Java class.
#
import gzip, re, sys, csv, os, os.path

assert(len(sys.argv) > 1)

# Comment line we are looking for
_headerre = re.compile(r"^# Data set size: (\d+) data type: DoubleVector,dim=(\d+)\s*$")

out = csv.writer(sys.stdout, delimiter=" ", lineterminator="\n")
out.writerow(["Name", "Group", "Semantic", "Size", "Dimensionality", "Outliers", "Rate"])
for infn in sys.argv[1:]:
	postfix=".raw.gz"
	assert(infn.endswith(postfix))
	if not os.path.exists(infn) and os.environ.get("SOFTFAIL"):
		if not os.path.exists(infn[:-3]):
			print >>sys.stderr, "Skipping missing file", infn
			continue
		postfix = ".raw"
		infn = infn[:-3] # Try non-compressed file
	nam = os.path.basename(infn[:-len(postfix)])
	grp = os.path.basename(os.path.dirname(infn))
	inf = gzip.open(infn) if infn.endswith(".gz") else open(infn)
	header = inf.readline()
	m = _headerre.match(header)
	if not m:
		print >>sys.stderr, "Header did not match in", infn
		print >>sys.stderr, header.strip()
		sys.exit(1)
	dsize, dim = m.group(1), m.group(2)
	bylabel = inf.readline()
	assert(bylabel.startswith("bylabel "))
	outliers=len(filter(lambda x: float(x) == 1., bylabel.split(" ")[1:]))
	rate = outliers / float(dsize)
	semantic = "y" if "semantic/" in infn else "n"
	out.writerow([nam, grp, semantic, dsize, dim, outliers, rate])
